var PACKAGE_NAME = "openalpraxis";
var LOCAL_DEBUG = false
var BASE_HOST = '' 
var BASE_URL = BASE_HOST + '/local/' + PACKAGE_NAME + '/openalpradmin.cgi'

//http://axisq.home/local/openalpraxis/openalpradmin.cgi?operation=get_disk_info

AlprFailureCodeEnum = {
    AGENT_NOT_STARTED : 0,
    MISSING_SDCARD : 1,
    INVALID_DATETIME : 2
}

var latest_stats = undefined;

function formatBytes(bytes) {
    if(bytes < 1024) return bytes + " Bytes";
    else if(bytes < 1048576) return(bytes / 1024).toFixed(2) + " KB";
    else if(bytes < 1073741824) return(bytes / 1048576).toFixed(2) + " MB";
    else return(bytes / 1073741824).toFixed(2) + " GB";
};


function show_failure(failure_codes)
{
    for (var i = 0; i < failure_codes.length; i++) {
        if (failure_codes[i] == AlprFailureCodeEnum.AGENT_NOT_STARTED)
        {
            // Show Agent not started
            $("#error_app_not_started").show();
        }
        else if (failure_codes[i] == AlprFailureCodeEnum.MISSING_SDCARD)
        {
            $("#error_missing_sdcard").show();
        }
        else if (failure_codes[i] == AlprFailureCodeEnum.INVALID_DATETIME)
        {
            $("#error_invalid_datetime").show();
        }
        //Do something
    }
    if (failure_codes.length > 0)
    {
        // Hide everything, only show failures
        $("div.main_setup_page").hide();
        $("div.error_page").show();
    }
}

function verify_config(config_data)
{
    JAN_1_2015_MS = 1420070400000;
    MIN_DISK_BYTES = 56000000; // 256MB
    // Checks the config data for validation errors.  If there are any, it shows the error text pages and returns false
    // If all is well, returns true
    error_list = []

    if (config_data.storage.available == false)
        error_list.push(AlprFailureCodeEnum.MISSING_SDCARD)
    else if (config_data.storage.drivespace_total_bytes < MIN_DISK_BYTES)
        error_list.push(AlprFailureCodeEnum.MISSING_SDCARD)

    if (config_data.timestamp_epoch_ms < JAN_1_2015_MS)
        error_list.push(AlprFailureCodeEnum.INVALID_DATETIME)

    if (error_list.length > 0)
    {
        show_failure(error_list);
        return false;
    }

    if (config_data.is_registered == false)
    {

        $(".initial_setup_page").show();
        return false;
    }

    return true;
}


var diskusage_donutchart;

function update_diskusage_chart(openalpr_usage_bytes)
{
    diskusage_donutchart.data.datasets[0].data[0] = openalpr_usage_bytes;
    diskusage_donutchart.update();
}

function load_diskusage_chart(config_data)
{
    // Check for 
    // Disk usage
    var disk_usage_bytes = config_data.storage.drivespace_total_bytes - config_data.storage.drivespace_free_bytes;
    var disk_usage_other = disk_usage_bytes - config_data.storage.drivespace_openalpr_bytes;

    var diskusage_data = {
        datasets: [{ 
            data: [
                config_data.storage.drivespace_openalpr_bytes,
                disk_usage_other,
                config_data.storage.drivespace_total_bytes
            ],
            backgroundColor: [
                "#36A2EB",
                "#FF6384",
                "#E7E9ED"
            ],
            borderColor: "#999999",
            borderWidth: 1,
            label: 'Disk Usage' // for legend
        }],
        labels: [
            "OpenALPR Usage",
            "Other Usage",
            "Free Space"
        ]
    };
    var ctx = document.getElementById("diskusagepiechart").getContext("2d");


    diskusage_donutchart = new Chart(ctx, {
        type: 'doughnut',
        data: diskusage_data,
        options: {
            responsive: true,
            maintainAspectRatio: false,
            cutoutPercentage: 55,
            legend: {
                display: false
            },
            tooltips : {

                callbacks : { // HERE YOU CUSTOMIZE THE LABELS
                    label : function(tooltipItem, data) {
                        return data.labels[tooltipItem.index] + ": " + formatBytes(data.datasets[tooltipItem.datasetIndex].data[tooltipItem.index]);
                    }
                }

            },
        } 
    });
}

function display_camera_image(width, height)
{
    var img_url = '/axis-cgi/jpg/image.cgi?resolution=' + width + 'x' + height;
    $camimage = $("<img width=" + width + " height=" + height + " />");

    $("#motionevents").prepend($camimage);

    var last_load = 0;
    function reload_cam_image()
    {
        // Reload at a maximum of 2fps
        var MIN_DELAY_MS = 500;
        var sleep_time = 0;
        if (Date.now() - last_load < MIN_DELAY_MS)
            sleep_time = MIN_DELAY_MS - (Date.now() - last_load);
        setTimeout(function(){ 
            $camimage.attr('src', img_url + "&rand=" + Math.random());
            last_load = Date.now()
        }, sleep_time);
    }

    $camimage.load(reload_cam_image).error(reload_cam_image).attr('src', img_url);
    
}

function update_stats()
{
    var request_start = Date.now();
    var request_end = request_start;
    $.getJSON( BASE_URL, 
        {
            operation: 'get_stats'
        },
        function( data ) {
            // Add datapoint to motion event


            $("#purge_uploads_button").attr('disabled', (data.upload_queue_size < 1))

            update_diskusage_chart(data.upload_queue_size_bytes);

            request_end = Date.now();

            $("#uploadactivity var").text(data.upload_queue_size);
            latest_stats = data;
            if (data.motion_detected)
                $("#motionevents").addClass('active');
            else
                $("#motionevents").removeClass('active');

            // Update the motion detection bar
            var motion_percentage = data.motion_percentage;
            var motion_sensitivity = data.motion_sensitivity;

            var left_bar = Math.min(motion_percentage, motion_sensitivity);
            var right_bar = Math.max(0, motion_percentage - motion_sensitivity);

            $("#motionline .leftline").css('width', left_bar + '%');
            $("#motionline .rightline").css('width', right_bar + '%');
    })
    .fail(function(data) {
        latest_stats = undefined;
        $("#uploadactivity var").text('-');
        $("#motionevents").removeClass('active');
    })
    .always(function() {
        
        var MIN_DELAY_MS = 1000;
        var sleep_time = 0;
        if (request_end - request_start < MIN_DELAY_MS)
            sleep_time = MIN_DELAY_MS - (request_end - request_start);

        setTimeout(update_stats, sleep_time);
    });

}

function start_app()
{
    $("#loading_spinner").show();
    $("error_app_not_started").hide();
    $("#start_app_button").attr('disabled', true);

    timestamp_now_ms = new Date().getTime();
    var url = BASE_HOST + "/axis-cgi/applications/control.cgi?action=start&package=" + PACKAGE_NAME + "&timestamp=" + timestamp_now_ms;
    $.get( url, function( data ) {

      setTimeout(function() {
        $("#loading_spinner").hide();
        location.reload();
      }, 7000);
      
    })
    .fail(function() {
      alert( "Failed to start application" );
      $("error_app_not_started").show();
      $("#loading_spinner").hide();
      $("#start_app_button").attr('disabled', false);
    })
    .always(function() {
    });
}


function signin_connect()
{
    $(".sign_in_message").clearQueue().hide();
    // Validate inputs
    var email_address = $("#txtEmail").val();
    var password = $("#txtPassword").val();
    
    if (email_address.length <= 2 || !email_address.includes("@"))
    {
        $(".sign_in_message.error").text("Invalid e-mail address").fadeIn(250).delay(5000).fadeOut(500);
        return;
    }
    else if (password.length <= 3)
    {
        $(".sign_in_message.error").text("Invalid password").fadeIn(250).delay(5000).fadeOut(500);
        return;
    }

    // Try connecting w/ AJAX

    $("#signin_ok_button").attr("disabled", true);

    $("#loading_spinner").show();
    $.getJSON( BASE_URL, 
        {
            operation: 'register',
            username: email_address,
            password: password
        },
        function( data ) {
            console.log(data);
            if (data.hasOwnProperty('error'))
            {
                $(".sign_in_message.error").text(data.error);
                $(".sign_in_message.error").fadeIn(250).delay(5000).fadeOut(500);
            }
            else
            {
                $(".sign_in_message.success").text("Connection Successful. Click OK to continue.").fadeIn(250).delay(5000).fadeOut(500);
                $("#signin_ok_button").attr("disabled", false);
                $("#signin_connect_button").attr('disabled', true);
            }
    })
    .fail(function(data) {
      $(".sign_in_message.error").text("Failed to sign-in").fadeIn(250).delay(5000).fadeOut(500);
    })
    .always(function() {
        $("#loading_spinner").hide();
    });
}

function deregister()
{
    bootbox.confirm("Are you sure you wish to deregister from the OpenALPR service?  The camera will no longer send video data to OpenALPR to be processed.", 
        function(result){ 
            if (result === true)
            {
                $.getJSON( BASE_URL, 
                    {
                        operation: 'deregister'
                    },
                    function( data ) {
                        $("#update_settings_status").addClass("success").removeClass("error").text("Deregister Successful").fadeIn(250).delay(5000).fadeOut(500);
                        document.location.reload(true);
                })
                .fail(function(data) {
                  // Check the error_reason, and show to user.
                  $("#update_settings_status").removeClass("success").addClass("error").text("Failed to Deregister account").fadeIn(250).delay(5000).fadeOut(500);
                })
                .always(function() {

                });
            }
         });




}

function purge_uploads()
{

    bootbox.confirm("Are you sure you wish to purge the upload queue?  This will delete all videos that have not yet been uploaded to OpenALPR.", 
        function(result){ 
            if (result === true)
            {
                $.getJSON( BASE_URL, 
                    {
                        operation: 'purge_uploads'
                    },
                    function( data ) {
                        $("#update_settings_status").addClass("success").removeClass("error").text("Purge Initiated").fadeIn(250).delay(5000).fadeOut(500);
                })
                .fail(function(data) {
                  // Check the error_reason, and show to user.
                  $("#update_settings_status").removeClass("success").addClass("error").text("Failed to start purge").fadeIn(250).delay(5000).fadeOut(500);
                })
                .always(function() {

                });
            }
         });
    
}

function initialize_config_settings(config_data)
{
    $("#motion_sensitivity_slider").slider('setValue', config_data.motion_sensitivity);
    $("#upload_speed_limit_slider").slider('setValue', config_data.upload_speed_limit_kbytes);
    $("#frames_per_second_slider").slider('setValue', config_data.max_frames_per_second);
    $("#h264_compression_slider").slider('setValue', config_data.h264_compression_percent);

    if (config_data.h264_ideal_width == 1920 && config_data.h264_ideal_height == 1080)
        $("#resolution_select").val("1920x1080");
    else
        $("#resolution_select").val("1280x720");

    $("#resolution_select").change();
}

function push_config_settings()
{
    $.getJSON( BASE_URL, 
        {
            operation: 'update_parameters',
            motion_sensitivity: $("#motion_sensitivity_slider").slider('getValue'),
            upload_speed_limit_kbytes: $("#upload_speed_limit_slider").slider('getValue'),
            h264_compression_percent: $("#h264_compression_slider").slider('getValue'),
            max_frames_per_second: $("#frames_per_second_slider").slider('getValue'),
            ideal_resolution: $("#resolution_select").val()
        },
        function( data ) {
            $("#update_settings_status").addClass("success").removeClass("error").text("Parameters Updated").fadeIn(250).delay(5000).fadeOut(500);
    })
    .fail(function(data) {
      // Check the error_reason, and show to user.
      $("#update_settings_status").removeClass("success").addClass("error").text("Failed to update parameters").fadeIn(250).delay(5000).fadeOut(500);
    })
    .always(function() {

    });
    //update_settings_button
}

function initialize_main_app_page(config_data)
{
    $("#version_label").text("v" + config_data.agent_version)
    var speed_values = [50, 100, 200, 400, 750, 1000, 1500, 2000, 5000, 50000];
    var speed_positions = []
    for (var i = 0; i < speed_values.length; i++)
    {
        speed_positions.push((i / (speed_values.length-1)) * 100);
        //console.log((i / (speed_values.length-1)) * 100)
    }
    $("#upload_speed_limit_slider").slider({
        value: config_data.upload_speed_limit_kbytes,
        ticks: speed_values,
        ticks_positions: speed_positions,
        ticks_snap_bounds: 50000,
        formatter: function(value) {
            if (value >= speed_values[speed_values.length-1])
                return "Unlimited";
            if (value > 1000)
                return value / 1000 + " MBytes/sec";
            return value + " KBytes/sec";
        }
    });

    $("#motion_sensitivity_slider").slider({
        value: config_data.upload_speed_limit_kbytes,
        min: 1,
        max: 100,
        ticks: [0, 20, 40, 60, 80, 100],
        //ticks_labels: ['Very Sensitive', '', '', '', '', 'Not Sensitive'],
        ticks_snap_bounds: 100,
        formatter: function(value) {
            if (value >= 100)
                return "Huge movement";
            if (value >= 80)
                return "Large movement";
            if (value >= 60)
                return "Medium movement";
            if (value >= 40)
                return "Small movement";
            if (value >= 20)
                return "Tiny movement";

            return "Minuscule movement";
        }
    });

    $("#frames_per_second_slider").slider({
        value: config_data.max_frames_per_second,
        min: 5,
        max: 20,
        ticks: [5, 10, 15, 20],
        //ticks_labels: ['Very Sensitive', '', '', '', '', 'Not Sensitive'],
        ticks_snap_bounds: 20,
        formatter: function(value) {
            if (value >= 20)
                return "20 FPS";
            if (value >= 15)
                return "15 FPS";
            if (value >= 10)
                return "10 FPS";
            if (value >= 5)
                return "5 FPS";

            return "Unknown FPS";
        }
    });

    $("#h264_compression_slider").slider({
        value: config_data.h264_compression_percent,
        min: 30,
        max: 60,
        ticks: [30, 35, 40, 45, 50, 55, 60],
        //ticks_labels: ['Very Sensitive', '', '', '', '', 'Not Sensitive'],
        ticks_snap_bounds: 60,
        formatter: function(value) {
            if (value >= 60)
                return "High compression (60)";
            if (value >= 55)
                return "High Compression (55)";
            if (value >= 50)
                return "High Compression (50)";
            if (value >= 45)
                return "Moderate Compression (45)";
            if (value >= 40)
                return "Low Compression (40)";
            if (value >= 35)
                return "Low Compression (35)";
            if (value >= 30)
                return "Low Compression (30)";

            return "Unknown Compression";
        }
    });

    $("#resolution_select").change(function() {
        var resolution = $("#resolution_select").val();
        if (resolution == "1920x1080")
        {
            $("#frames_per_second_slider").slider('setAttribute', 'max', 10);
            $("#frames_per_second_slider").slider('setAttribute', 'ticks', [5, 10]);
        }
        else
        {
            $("#frames_per_second_slider").slider('setAttribute', 'max', 20);
            $("#frames_per_second_slider").slider('setAttribute', 'ticks', [5, 10, 15, 20]);
        }
        $("#frames_per_second_slider").slider('refresh', { useCurrentValue: true });
    });
    // consider cubism: https://square.github.io/cubism/
    sparkline_fps = 1;
    sparkline_history_seconds = 120;
    upload_sparkline_data = [];
    h264thoughput_sparkline_data = [];
    for (var i = 0; i < sparkline_fps * sparkline_history_seconds; i++)
    {
        upload_sparkline_data.push(0);
        h264thoughput_sparkline_data.push(0);
    }

    display_camera_image(config_data.motion_width, config_data.motion_height); 
    initialize_config_settings(config_data);
    update_stats();

    setInterval(function(){
        var latest_upload_speed = 0;
        var latest_h264throughput = 0;
        if (latest_stats !== undefined)
        {
            latest_upload_speed = latest_stats.upload_speed_bytes / 1024;
            latest_h264throughput = latest_stats.h264_throughput_bytes_per_second / 1024;
        }


        // Update the upload sparkline
        upload_sparkline_data.push(parseInt(Math.round(latest_upload_speed)));
        upload_sparkline_data.shift();

        h264thoughput_sparkline_data.push(parseInt(Math.round(latest_h264throughput)));
        h264thoughput_sparkline_data.shift();

        $("#videothroughput var").text(parseInt(latest_h264throughput));
        // var MOVING_AVERAGE_SECONDS = 5;
        // var sum = 0;
        // for( var i = upload_sparkline_data.length-1; i >= upload_sparkline_data.length-MOVING_AVERAGE_SECONDS; i-- ){
        //   sum += parseInt( upload_sparkline_data[i], 10 ); //don't forget to add the base
        // }
        // var avg = Math.round(sum / MOVING_AVERAGE_SECONDS);
        //$("#uploadactivity var").text(avg);

        $("#uploadactivity .linechart").sparkline(upload_sparkline_data, {
            type: 'line',
            lineWidth: 1,
            fillColor: false,
            lineColor: '#36A2EB',
            tooltipSuffix: ' kB/s',
            highlightSpotColor: null,
            highlightLineColor: null,
            spotColor: false,
            minSpotColor: false,
            maxSpotColor: false,
            highlightSpotColor: false,
            highlightLineColor: false,
            chartRangeMin: 0,
            height: '30px',
            width: '100%'
        });

        $("#videothroughput .linechart").sparkline(h264thoughput_sparkline_data, {
            type: 'line',
            lineWidth: 1,
            fillColor: false,
            lineColor: '#d9534f',
            tooltipSuffix: ' kB/s',
            highlightSpotColor: null,
            highlightLineColor: null,
            spotColor: false,
            minSpotColor: false,
            maxSpotColor: false,
            highlightSpotColor: false,
            highlightLineColor: false,
            chartRangeMin: 0,
            height: '30px',
            width: '100%'
        });

    }, 1000 / sparkline_fps);
}


function wait_for_app_to_start()
{
  // Poll regularly to see if app has started
  setInterval(function(){
    $.getJSON( BASE_URL, {
        operation: 'get_config'
    }, function( data ) {
      if (verify_config(data) === true)
      {
        location.reload();
      }
    });
  }, 3000);
}

$(function(){

    $.getJSON( BASE_URL, {
        operation: 'get_config'
    }, function( data ) {
      if (verify_config(data) === true)
      {
        $(".main_setup_page").show();
        load_diskusage_chart(data);
        initialize_main_app_page(data);
      }
    })
    .fail(function() {
      show_failure([AlprFailureCodeEnum.AGENT_NOT_STARTED]);
      wait_for_app_to_start();
    })
    .always(function() {
        $("#loading_spinner").hide();
    });

    $("#start_app_button").click(start_app);
    $("#purge_uploads_button").click(purge_uploads);
    $("#signin_connect_button").click(signin_connect);
    $("#signin_ok_button").click(function() { location.reload(); });
    $("#disconnect_user_button").click(deregister);
    $("#update_settings_button").click(push_config_settings);
});
